package de.ozgcloud.eingang.fim;

import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.IncomingFile;
import de.ozgcloud.eingang.common.formdata.IncomingFileGroup;
import de.ozgcloud.eingang.semantik.enginebased.EngineBasedMapper;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.IOException;
import java.util.Collection;
import java.util.List;
import java.util.Optional;

@Log4j2
@Component
class FimBasedMapper implements EngineBasedMapper {

    @Autowired
    private FimService fimService;

    public boolean isResponsible(final FormData formData) {
        //FIXME isResponsible from meta data
        final List<IncomingFileGroup> attachments = formData.getAttachments();
        final Optional<IncomingFile> metadataFileOpt = findFile(attachments, "fim_xtaMetadata.xml");
        return metadataFileOpt.isPresent();
    }

    public FormData parseFormData(final FormData initialFormData) {
        if (!isResponsible(initialFormData)) {
            return initialFormData;
        }

        final List<IncomingFileGroup> attachments = initialFormData.getAttachments();
        final Optional<IncomingFile> metadataFileOpt = findFile(attachments, "fim_xtaMetadata.xml");
        if (metadataFileOpt.isEmpty()) {
            LOG.error("Metadata File not found for fim data mapping");
            return initialFormData;
        }
        final Optional<String> entryPointOpt = getEntryPoint(metadataFileOpt.get());
        if (entryPointOpt.isEmpty()) {
            LOG.error("No entry point found in metadata file for fim data mapping");
            return initialFormData;
        }
        final Optional<IncomingFile> fileOpt = findFile(attachments, entryPointOpt.get());
        if (fileOpt.isEmpty()) {
            LOG.error("Entry point file not found for fim data mapping");
            return initialFormData;
        }
        try {
            final Document document = loadDocument(fileOpt.get());
            return fimService.transformDocument(document, initialFormData);
        } catch (ParserConfigurationException | SAXException | IOException | FimException e) {
            LOG.error("Can't transform document into fim formdata");
            return initialFormData;
        }
    }

    private Optional<IncomingFile> findFile(final Collection<IncomingFileGroup> attachments, final String name) {
        for (IncomingFileGroup group : attachments) {
            for (IncomingFile file : group.getFiles()) {
                if (file.getName().endsWith(name)) {
                    return Optional.of(file);
                }
            }
        }
        return Optional.empty();
    }

    private Optional<String> getEntryPoint(final IncomingFile incomingFile) {
        // FIXME sollte über formdata metadaten kommen statt aus dem meta file geparsed zu werden
        return Optional.of("Antrag.xml");
    }

    private Document loadDocument(final IncomingFile incomingFile) throws ParserConfigurationException, IOException, SAXException {
        final DocumentBuilder builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
        return builder.parse(incomingFile.getContentStream());
    }
}
