package de.ozgcloud.eingang.semantik.enginebased.afm.intelliform;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;

import de.ozgcloud.common.errorhandling.TechnicalException;
import de.ozgcloud.eingang.semantik.enginebased.afm.intelliform.JsonService;
import lombok.SneakyThrows;

class JsonServiceTest {

	@InjectMocks
	private JsonService service;
	@Mock
	private ObjectMapper objectMapper;

	@DisplayName("Map form data")
	@Nested
	class TestMapFormData {

		@DisplayName("with invalid json")
		@Nested
		class TestWithInvalidJson {

			@SneakyThrows
			@BeforeEach
			void mockNode() {
				when(objectMapper.readValue(anyString(), Mockito.<TypeReference<List<Map<String, Object>>>>any()))
						.thenThrow(JsonProcessingException.class);
			}

			@SneakyThrows
			@Test
			void shouldCallObjectMapper() {
				try {
					mapFormData();
				} catch (TechnicalException e) {
					verify(objectMapper).readValue(anyString(), Mockito.<TypeReference<List<Map<String, Object>>>>any());
				}
			}

			@Test
			void shouldThrowTechnicalException() {
				assertThatThrownBy(() -> mapFormData())
						.isInstanceOf(TechnicalException.class)
						.hasMessageStartingWith("Error parsing JSON")
						.hasMessageContaining("ExceptionId");
			}
		}

		@DisplayName("with valid json")
		@Nested
		class TestWithValidJson {

			@SneakyThrows
			@BeforeEach
			void mockNode() {
				when(objectMapper.readValue(anyString(), Mockito.<TypeReference<List<Map<String, Object>>>>any()))
						.thenReturn(Collections.emptyList());
			}

			@SneakyThrows
			@Test
			void shouldCallObjectMapper() {
				mapFormData();

				verify(objectMapper).readValue(anyString(), Mockito.<TypeReference<List<Map<String, Object>>>>any());
			}

			@Test
			void shouldReturnValue() {
				var list = mapFormData();

				assertThat(list).isEmpty();
			}
		}

		private List<Map<String, Object>> mapFormData() {
			return service.readAsListMap("{}");
		}
	}
}
